function [F,DF,Fext,a2ext,DFlambda] = FDF(a,lambda,n)
% [F,DF,Fext,a2ext,DFlambda] = FDF(a,lambda,n)
% evaluates 
% D^2 u + lambda (u-u^3) 
% where D^2 is the laplacian and lambda is a parameter
% Here u is represented by a 2D cosine series with Fourier coefficients
% encoded in the (n(1)+1) by (n(2)+1) matrix A
% u = sum_{k=-n}^{n} A(|k_1|,|k_2|) exp(i (k_1 x_1 + k_2 x_2))
% where n=(n(1),n(2))
%
% Note that the input a is in fact the vector a=A(:) 
% The output F is a vector of length (n(1)+1)(n(2)+1)
% and DF is the Jacobian matrix of size (n(1)+1)(n(2)+1) by (n(1)+1)(n(2)+1)
% Fext is the matrix of size (3n(1)+1) by (3n(2)+1) representing all
% nonzero elements of the evaluation.
% a2ext is the matrix of size (4n(1)+1) by (4n(2)+1) representing u^2
% in terms of Fourier coefficients
% and DFlambda is the derivative of F with respect to lambda (as a vector)

a=reshape(a(:),n+1);
[kx,ky]=ndgrid(0:n(1),0:n(2));

% factors to go from cosine series to Fourier series (m_k in the pdf)
multiext=4*ones(3*n+1);
multiext(:,1)=2;
multiext(1,:)=2;
multiext(1,1)=1;
multi=multiext(1:n(1)+1,1:n(2)+1);

% nonlinearities
[a2,a2ext]=conv2cos(a,a);
a3=conv2cos(a,a2);

% linear part
symbol=-kx.^2-ky.^2 +lambda;

% compute full F
Fext=-lambda*a3;
Fext(1:n(1)+1,1:n(2)+1)=Fext(1:n(1)+1,1:n(2)+1)+symbol.*a;
Fext=multiext.*Fext;

% reduced F
F=Fext(1:n(1)+1,1:n(2)+1);
F=F(:);

%%%%% Jacobian %%%%%

Nxy=(n(1)+1)*(n(2)+1);
DF=altzeros([n+1,n+1],a(1));

Ix=2*n(1)+(1:n(1)+1);
Iy=2*n(2)+(1:n(2)+1);

% convolution with a*a
mat0=altzeros(n+1,a(1));
for i=1:length(kx(:))
   mat=mat0;
   for jx=[-kx(i) kx(i)]
      for jy=[-ky(i) ky(i)]
          mat=mat+a2ext(Ix-jx,Iy-jy);
      end
   end 
   % remove duplications
   if kx(i)==0
      mat=mat/2;
   end
   if ky(i)==0
      mat=mat/2;
   end
   % put in m_k factor in definition of F
   DF(:,:,kx(i)+1,ky(i)+1)=multi.*mat;
end

% reshape as a matrix
DF=reshape(DF,Nxy,Nxy);
% multiply by -3*lambda and add linear term on the diagional
DF=-3*lambda*DF+diag(symbol(:).*multi(:));

% derivative w.r.t. lambda
DFlambda=a-a3(1:n(1)+1,1:n(2)+1);
DFlambda=multi.*DFlambda;
DFlambda=DFlambda(:);

end

