function [success,rmin,rmax,morse] = proveit(a,lambda,n,nu,m)
% attempts to construct a proof in the ball around a
% lambda is assumed positive
% if m is provided a is first padded with zeros to size m
% If lambda is an interval, then the proof will be a proper proof
% including interval arithmetic.

% details are provided in the file estimates-2D.pdf

success=0;
rmin=NaN;
rmax=NaN;
morse=NaN;

if exist('intval','file') && isintval(lambda)
  disp('including interval arithmetic')
  a=intval(a);
  nu=intval(nu);
else
  disp('no intervals yet')
end

if lambda<= 0
    disp('failure: lambda needs to be positive')
    return
end

if exist('m','var') 
  if m(1)>=n(1) && m(2)>=n(2)
    % zero padding
    anew=altzeros(m+1,a(1));   
    anew(1:n(1)+1,1:n(2)+1)=reshape(a,n+1);
    a=anew;
    n=m;  
  else
    disp('failure: m must be chosen at least as large as n')
    return
  end
end

% minimum number of modes
if min(n+1)^2>lambda 
   % called C_K in the pdf 
   normAMtail=1/(min(n+1)^2-lambda);
else
   disp('failure: the dimension is not large enough (smaller than sqrt(lambda))')
   return
end

% get F(a) and DF(a) as well as a*a
[F,DF,Fext,a2ext]=FDF(a,lambda,n);

% determine A
if exist('intval','file') && isintval(lambda)
    A=intval(inv(mid(DF)));
else
    A=inv(DF);
end 

% reshape a to be 2-dimensional
a=reshape(a(:),n+1);

% mode number up to 3 times the size of a
[EEkx,EEky]=ndgrid(0:3*n(1),0:3*n(2));

% the differential operator
EEsymbol=-EEkx.^2-EEky.^2 +lambda;

% finite part 
finite=(EEkx<=n(1) & EEky<=n(2));
% intermediate part
Efinite=(EEkx<=2*n(1) & EEky<=2*n(2));
% the tail
notfinite=find(~finite);

% multiplicities in the weights
multi=4*ones(size(EEkx));
multi(EEkx==0)=multi(EEkx==0)/2;
multi(EEky==0)=multi(EEky==0)/2;
EEweights=multi.*nu.^(max(abs(EEkx),abs(EEky)));
weights=EEweights(finite);
Eweights=EEweights(Efinite);

% bound on the operator norm of A*M
multismall=multi(1:n(1)+1,1:n(2)+1);
AM=A.*diag(multismall(:));
normAMfinite=max((weights'*abs(AM))./weights');
normAM=max(normAMfinite,normAMtail);

% bound Y0
AFfinite=A*F;
AF=Fext./(EEsymbol.*multi);
AF(finite)=AFfinite;
Y0=sum(abs(AF(:)).*EEweights(:));

% bound Z0
B=eye((n(1)+1)*(n(2)+1))-A*DF;
Z0=max((weights'*abs(B))./weights');

% bound Z1

% pad the Fourier coefficients of a*a with zeros
Ea2=altzeros(8*n+1,a(1));
Ea2(2*n(1)+1:6*n(1)+1,2*n(2)+1:6*n(2)+1)=a2ext;
Ix=4*n(1)+(1:n(1)+1);
Iy=4*n(2)+(1:n(2)+1);

% the main part of the Z1 bound
mat0=altzeros(n+1,a(1));
% Pmax will contain tilde{P}_k for all k<=n
Pmax=mat0;
for j=length(notfinite)
   i=notfinite(j);
   mat=mat0;
   for jx=[-EEkx(i) EEkx(i)]
      for jy=[-EEky(i) EEky(i)]
          mat=mat+Ea2(Ix-jx,Iy-jy);
      end
   end 
   mat=(multi(j)*multi(i)/(4*EEweights(i)))*abs(mat);
   Pmax=max(Pmax,mat);
end

% finalizing the Z1 bound
AP=abs(A)*Pmax(:);
% the cosine coefficients of a*a
a2=a2ext(2*n(1)+1:4*n(1)+1,2*n(2)+1:4*n(2)+1);
norma2=sum(abs(a2(:)).*Eweights);
Z1=3*lambda*(sum(AP.*weights)+normAMtail*norma2);

% bound Z2 
norma=sum(abs(a(:)).*weights);
Z2=6*lambda*normAM*norma;

% bound Z3 
Z3=3*lambda*normAM;

% some checks to see if the proof could work (ignoring Z3)
slope=-1+Z0+Z1;
if ~(slope<0)
     disp('failure: slope is positive') 
     disp(altsup([Y0,Z0,Z1,Z2]))
     return
end
discr=(-1+Z0+Z1)^2-4*(Y0*Z2);
if ~(discr>0)
     disp('failure: discrimant is negative') 
     disp(['discriminant is ',num2str(-altsup(-discr))])
     disp(altsup([Y0,Z0,Z1,Z2]))
     return
end

% numerical roots
r0=roots(altsup([Z3,Z2,-1+Z1+Z0,Y0]));

% merge Z2 and Z3 by using an a priori bound r_0 on r: Z2+Z3*r <= Z2+Z3*r0
r0=1.1*abs(max(real(r0)));
Z2r0=Z2+Z3*r0;

% use quadratic formula for the resulting parabola
discr=(-1+Z0+Z1)^2-4*(Y0*Z2r0);
if discr>0 
    if altsup((-slope-sqrt(discr))/(2*Z2r0)) <r0
        rmin=altsup((-slope-sqrt(discr))/(2*Z2r0));
        rmax=min(-altsup(-((-slope+sqrt(discr))/(2*Z2r0))),r0);
        success=1;
        disp('success: we have a proof!') 
    else
        disp('failure: problem with r0') 
    end
else
    disp('failure: discriminant is negative') 
end

% compute the Morse index of the matrix DF
if success
    morse=countunstable(DF);
    if ~isnan(morse)
        disp(['(relative) Morse index is ',int2str(morse)]);
    else
        disp('(relative) Morse index could not be determined')
    end
end

end

